@@ -17,6 +17,7 @@ def file_get_contents(filename, encoding=None):
 
 kem_yamls = []
 sig_yamls = []
+sig_stfl_yamls = []
 
 ########################################
 # Update the KEM markdown documentation.
@@ -47,6 +48,12 @@ def do_it(liboqs_root):
                     out_md.write('  - **{}**:<a name="{}"></a>\n'.format(opt_upstream, opt_upstream))
                     out_md.write('      - **Source**: {}\n'.format(kem_yaml['optimized-upstreams'][opt_upstream]['source']))
                     out_md.write('      - **Implementation license (SPDX-Identifier)**: {}\n'.format(kem_yaml['optimized-upstreams'][opt_upstream]['spdx-license-identifier']))
+            if 'formally-verified-upstreams' in kem_yaml:
+                out_md.write('- **Formally-verified Implementation sources**: \n')
+                for opt_upstream in kem_yaml['formally-verified-upstreams']:
+                    out_md.write('  - **{}**:<a name="{}"></a>\n'.format(opt_upstream, opt_upstream))
+                    out_md.write('      - **Source**: {}\n'.format(kem_yaml['formally-verified-upstreams'][opt_upstream]['source']))
+                    out_md.write('      - **Implementation license (SPDX-Identifier)**: {}\n'.format(kem_yaml['formally-verified-upstreams'][opt_upstream]['spdx-license-identifier']))
             if 'upstream-ancestors' in kem_yaml:
                 out_md.write('- **Ancestors of primary source**:\n')
                 for url in kem_yaml['upstream-ancestors'][:-1]:
@@ -68,7 +75,8 @@ def do_it(liboqs_root):
                       'Public key size (bytes)',
                       'Secret key size (bytes)',
                       'Ciphertext size (bytes)',
-                      'Shared secret size (bytes)']]
+                      'Shared secret size (bytes)',
+                      'Keypair seed size (bytes)']]
             for parameter_set in kem_yaml['parameter-sets']:
                 table.append([parameter_set['name'],
                               parameter_set['alias'] if 'alias' in parameter_set else "NA",
@@ -77,12 +85,13 @@ def do_it(liboqs_root):
                               parameter_set['length-public-key'],
                               parameter_set['length-secret-key'],
                               parameter_set['length-ciphertext'],
-                              parameter_set['length-shared-secret']])
+                              parameter_set['length-shared-secret'],
+                              parameter_set['length-keypair-seed'] if 'length-keypair-seed' in parameter_set else "NA"])
             out_md.write(tabulate.tabulate(table, tablefmt="pipe", headers="firstrow", colalign=("center",)))
             out_md.write('\n')
 
             for index, parameter_set in enumerate(kem_yaml['parameter-sets']):
-                out_md.write('\n## {} implementation characteristics\n\n'.format(parameter_set['name'].replace("_", "\_")))
+                out_md.write('\n## {} implementation characteristics\n\n'.format(parameter_set['name'].replace("_", "\\_")))
                 table_header = ['Implementation source',
                                 'Identifier in upstream',
                                 'Supported architecture(s)',
@@ -101,7 +110,7 @@ def do_it(liboqs_root):
                     # if all platforms are supported, assuming not optimized and is primary upstream
                     if impl['supported-platforms'] == 'all':
                         table.append(['[Primary Source](#primary-source)',
-                                      impl['upstream-id'].replace('_', '\_'),
+                                      impl['upstream-id'].replace('_', '\\_'),
                                       'All',
                                       'All',
                                       'None',
@@ -125,8 +134,8 @@ def do_it(liboqs_root):
                                 anchor = impl['upstream']
                             upstream_name = '[{}](#{})'.format(name, anchor)
                             table.append([upstream_name,
-                                          impl['upstream-id'].replace('_', '\_'),
-                                          platform['architecture'].replace('_', '\_'),
+                                          impl['upstream-id'].replace('_', '\\_'),
+                                          platform['architecture'].replace('_', '\\_'),
                                           op_systems,
                                           flags,
                                           impl['no-secret-dependent-branching-claimed'],
@@ -195,7 +204,7 @@ def do_it(liboqs_root):
                       'Secret key size (bytes)',
                       'Signature size (bytes)']]
             for parameter_set in sig_yaml['parameter-sets']:
-                table.append([parameter_set['name'].replace('_', '\_'),
+                table.append([parameter_set['name'].replace('_', '\\_'),
                               parameter_set['alias'] if 'alias' in parameter_set else "NA",
                               parameter_set['claimed-security'],
                               parameter_set['claimed-nist-level'],
@@ -206,7 +215,7 @@ def do_it(liboqs_root):
             out_md.write('\n')
 
             for index, parameter_set in enumerate(sig_yaml['parameter-sets']):
-                out_md.write('\n## {} implementation characteristics\n\n'.format(parameter_set['name'].replace("_", "\_")))
+                out_md.write('\n## {} implementation characteristics\n\n'.format(parameter_set['name'].replace("_", "\\_")))
                 table_header = ['Implementation source',
                                 'Identifier in upstream',
                                 'Supported architecture(s)',
@@ -225,7 +234,7 @@ def do_it(liboqs_root):
                     # if all platforms are supported, assuming not optimized and is primary upstream
                     if impl['supported-platforms'] == 'all':
                         table.append(['[Primary Source](#primary-source)',
-                                      impl['upstream-id'].replace('_', '\_'),
+                                      impl['upstream-id'].replace('_', '\\_'),
                                       'All',
                                       'All',
                                       'None',
@@ -249,8 +258,8 @@ def do_it(liboqs_root):
                                 anchor = impl['upstream']
                             upstream_name = '[{}](#{})'.format(name, anchor)
                             table.append([upstream_name,
-                                          impl['upstream-id'].replace('_', '\_'),
-                                          platform['architecture'].replace('_', '\_'),
+                                          impl['upstream-id'].replace('_', '\\_'),
+                                          platform['architecture'].replace('_', '\\_'),
                                           op_systems,
                                           flags,
                                           impl['no-secret-dependent-branching-claimed'],
@@ -269,6 +278,66 @@ def do_it(liboqs_root):
             out_md.write('- **Large Stack Usage**: Implementations identified as having such may cause failures when running in threads or in constrained environments.')
 
 
+    ##############################################
+    # Update the stateful signature markdown documentation.
+    ##############################################
+    for sig_stfl_yaml_path in sorted(glob.glob(os.path.join(liboqs_root, 'docs', 'algorithms', 'sig_stfl', '*.yml'))):
+        sig_stfl_yaml = load_yaml(sig_stfl_yaml_path)
+        sig_stfl_yamls.append(sig_stfl_yaml)
+        sig_stfl_name = os.path.splitext(os.path.basename(sig_stfl_yaml_path))[0]
+        print('Updating {}/{}.md'.format(os.path.dirname(sig_stfl_yaml_path), sig_stfl_name))
+
+        with open(os.path.join(liboqs_root, 'docs', 'algorithms', 'sig_stfl', '{}.md'.format(sig_stfl_name)), mode='w', encoding='utf-8') as out_md:
+            out_md.write('# {}\n\n'.format(sig_stfl_yaml['name']))
+            out_md.write('- **Algorithm type**: Digital signature scheme.\n')
+            out_md.write('- **Main cryptographic assumption**: {}.\n'.format(sig_stfl_yaml['crypto-assumption']))
+            out_md.write('- **Principal submitters**: {}.\n'.format(', '.join(sig_stfl_yaml['principal-submitters'])))
+            if 'auxiliary-submitters' in sig_stfl_yaml and sig_stfl_yaml['auxiliary-submitters']:
+                out_md.write('- **Auxiliary submitters**: {}.\n'.format(', '.join(sig_stfl_yaml['auxiliary-submitters'])))
+            out_md.write('- **Authors\' website**: {}\n'.format(sig_stfl_yaml['website']))
+            out_md.write('- **Specification version**: {}.\n'.format(sig_stfl_yaml['spec-version']))
+
+            out_md.write('- **Primary Source**<a name="primary-source"></a>:\n')
+            out_md.write('  - **Source**: {}\n'.format(sig_stfl_yaml['primary-upstream']['source']))
+            out_md.write('  - **Implementation license (SPDX-Identifier)**: {}\n'.format(sig_stfl_yaml['primary-upstream']['spdx-license-identifier']))
+            if 'optimized-upstreams' in sig_stfl_yaml:
+                out_md.write('- **Optimized Implementation sources**: {}\n'.format(sig_stfl_yaml['primary-upstream']['source']))
+                for opt_upstream in sig_stfl_yaml['optimized-upstreams']:
+                    out_md.write('  - **{}**:<a name="{}"></a>\n'.format(opt_upstream, opt_upstream))
+                    out_md.write('      - **Source**: {}\n'.format(sig_stfl_yaml['optimized-upstreams'][opt_upstream]['source']))
+                    out_md.write('      - **Implementation license (SPDX-Identifier)**: {}\n'.format(sig_stfl_yaml['optimized-upstreams'][opt_upstream]['spdx-license-identifier']))
+
+            if 'upstream-ancestors' in sig_stfl_yaml:
+                out_md.write(', which takes it from:\n')
+                for url in sig_stfl_yaml['upstream-ancestors'][:-1]:
+                    out_md.write('  - {}, which takes it from:\n'.format(url))
+                out_md.write('  - {}\n'.format(sig_stfl_yaml['upstream-ancestors'][-1]))
+            else:
+                out_md.write('\n')
+
+            if 'advisories' in sig_stfl_yaml:
+                out_md.write('\n## Advisories\n\n')
+                for advisory in sig_stfl_yaml['advisories']:
+                    out_md.write('- {}\n'.format(advisory))
+
+            out_md.write('\n## Parameter set summary\n\n')
+            table = [['Parameter set',
+                      'Security model',
+                      'Claimed NIST Level',
+                      'Public key size (bytes)',
+                      'Secret key size (bytes)',
+                      'Signature size (bytes)']]
+            for parameter_set in sig_stfl_yaml['parameter-sets']:
+                table.append([parameter_set['name'],
+                              parameter_set['claimed-security'],
+                              parameter_set['claimed-nist-level'],
+                              parameter_set['length-public-key'],
+                              parameter_set['length-secret-key'],
+                              parameter_set['length-signature']])
+            out_md.write(tabulate.tabulate(table, tablefmt="pipe", headers="firstrow", colalign=("center",)))
+            out_md.write('\n')
+
+
 
     ####################
     # Update the README.
@@ -326,40 +395,43 @@ def do_it(liboqs_root):
     with open(readme_path, mode='w', encoding='utf-8') as readme:
         readme.write(preamble + identifier_start + '\n')
 
-        for sig_yaml in sig_yamls[:-1]: # SPHINCS is last in this sorted list and requires special handling.
+        for sig_yaml in sig_yamls:
+            # SPHINCS requires special handling.
+            if "SPHINCS" in sig_yaml["name"]:
+                for hash_func in ['SHA2', 'SHAKE']:
+                    parameter_sets = [pset for pset in sig_yaml['parameter-sets'] if hash_func in pset['name']]
+                    if any(impl['large-stack-usage'] for impl in parameter_sets[0]['implementations']):
+                        readme.write('- **SPHINCS+-{}**: {}†'.format(hash_func, parameter_sets[0]['name'].replace('_','\\_')))
+                    else:
+                        readme.write('- **SPHINCS+-{}**: {}'.format(hash_func, parameter_sets[0]['name'].replace('_','\\_')))
+                    for parameter_set in parameter_sets[1:]:
+                        if any(impl['large-stack-usage'] for impl in parameter_set['implementations']):
+                            readme.write(', {}†'.format(parameter_set['name'].replace('_', '\\_')))
+                        else:
+                            readme.write(', {}'.format(parameter_set['name'].replace('_', '\\_')))
+                    readme.write('\n')
+                continue
+            
             parameter_sets = sig_yaml['parameter-sets']
             if any(impl['large-stack-usage'] for impl in parameter_sets[0]['implementations']):
-                readme.write('- **{}**: {}†'.format(sig_yaml['name'], parameter_sets[0]['name'].replace('_','\_')))
+                readme.write('- **{}**: {}†'.format(sig_yaml['name'], parameter_sets[0]['name'].replace('_','\\_')))
                 if 'alias' in parameter_sets[0]:
-                    readme.write(' (alias: {})'.format(parameter_sets[0]['alias']).replace('_','\_'))
+                    readme.write(' (alias: {})'.format(parameter_sets[0]['alias']).replace('_','\\_'))
             else:
-                readme.write('- **{}**: {}'.format(sig_yaml['name'], parameter_sets[0]['name'].replace('_','\_')))
+                readme.write('- **{}**: {}'.format(sig_yaml['name'], parameter_sets[0]['name'].replace('_','\\_')))
                 if 'alias' in parameter_sets[0]:
-                    readme.write(' (alias: {})'.format(parameter_sets[0]['alias']).replace('_','\_'))
+                    readme.write(' (alias: {})'.format(parameter_sets[0]['alias']).replace('_','\\_'))
             for parameter_set in parameter_sets[1:]:
                 if any(impl['large-stack-usage'] for impl in parameter_set['implementations']):
-                    readme.write(', {}†'.format(parameter_set['name'].replace('_', '\_')))
+                    readme.write(', {}†'.format(parameter_set['name'].replace('_', '\\_')))
                     if 'alias' in parameter_set:
-                        readme.write(' (alias: {})'.format(parameter_set['alias']).replace('_','\_'))
+                        readme.write(' (alias: {})'.format(parameter_set['alias']).replace('_','\\_'))
                 else:
-                    readme.write(', {}'.format(parameter_set['name'].replace('_', '\_')))
+                    readme.write(', {}'.format(parameter_set['name'].replace('_', '\\_')))
                     if 'alias' in parameter_set:
-                        readme.write(' (alias: {})'.format(parameter_set['alias']).replace('_','\_'))
+                        readme.write(' (alias: {})'.format(parameter_set['alias']).replace('_','\\_'))
             readme.write('\n')
 
-        sphincs_yml = sig_yamls[-1]
-        for hash_func in ['SHA2', 'SHAKE']:
-            parameter_sets = [pset for pset in sphincs_yml['parameter-sets'] if hash_func in pset['name']]
-            if any(impl['large-stack-usage'] for impl in parameter_sets[0]['implementations']):
-                readme.write('- **SPHINCS+-{}**: {}†'.format(hash_func, parameter_sets[0]['name'].replace('_','\_')))
-            else:
-                readme.write('- **SPHINCS+-{}**: {}'.format(hash_func, parameter_sets[0]['name'].replace('_','\_')))
-            for parameter_set in parameter_sets[1:]:
-                if any(impl['large-stack-usage'] for impl in parameter_set['implementations']):
-                    readme.write(', {}†'.format(parameter_set['name'].replace('_', '\_')))
-                else:
-                    readme.write(', {}'.format(parameter_set['name'].replace('_', '\_')))
-            readme.write('\n')
 
         readme.write(postamble)
 
